      SUBROUTINE soil(indice,
     s                ptimestep, pctsrf, snow, ptsrf, ptsoil,
     s                pcapcal, pfluxgrd)
      IMPLICIT NONE

c=======================================================================
c
c   Auteur:  Frederic Hourdin     30/01/92
c   -------
c
c   objet:  computation of : the soil temperature evolution
c   ------                   the surfacic heat capacity "Capcal"
c                            the surface conduction flux pcapcal
c
c
c   Method: implicit time integration
c   -------
c   Consecutive ground temperatures are related by:
c           T(k+1) = C(k) + D(k)*T(k)  (1)
c   the coefficients C and D are computed at the t-dt time-step.
c   Routine structure:
c   1)new temperatures are computed  using (1)
c   2)C and D coefficients are computed from the new temperature
c     profile for the t+dt time-step
c   3)the coefficients A and B are computed where the diffusive
c     fluxes at the t+dt time-step is given by
c            Fdiff = A + B Ts(t+dt)
c     or     Fdiff = F0 + Capcal (Ts(t+dt)-Ts(t))/dt
c            with F0 = A + B (Ts(t))
c                 Capcal = B*dt
c           
c   Interface:
c   ----------
c
c   Arguments:
c   ----------
c   ptimestep            physical timestep (s)
c   snow                 snow
c   ptsrf                surface temperature at time-step t (K)
c   ptsoil               temperature inside the ground (K)
c   pcapcal              surfacic specific heat (W*m-2*s*K-1)
c   pfluxgrd             surface diffusive flux from ground (Wm-2)
c   
c=======================================================================
c   declarations:
c   -------------

#include "dimensions.h"
#include "dimphy.h"
#include "dimsoil.h"
#include "indicesol.h"

c-----------------------------------------------------------------------
c  arguments
c  ---------

      INTEGER indice
      REAL ptimestep
      REAL pctsrf(klon,nbsrf)
      REAL ptsrf(klon,nbsrf)
      REAL ptsoil(klon,nsoilmx,nbsrf)
      REAL snow(klon,nbsrf)
      REAL pcapcal(klon,nbsrf)
      REAL pfluxgrd(klon,nbsrf)

c-----------------------------------------------------------------------
c  local arrays
c  ------------

      INTEGER ig,jk, nsrf
      REAL zdz2(nsoilmx),z1
      REAL min_period,dalph_soil
      REAL ztherm_i(klon,nbsrf)
c

c   local saved variables:
c   ----------------------
      REAL dz1(nsoilmx),dz2(nsoilmx)
      SAVE dz1,dz2
      REAL lambda
      SAVE lambda
      REAL zc(klon,nsoilmx,nbsrf),zd(klon,nsoilmx,nbsrf)
      SAVE zc,zd
      LOGICAL firstcall
      SAVE firstcall
      REAL isol,isno,iice
      SAVE isol,isno,iice

      DATA firstcall/.true./

      DATA isol,isno,iice/2000.,2000.,2000./

c-----------------------------------------------------------------------
c   Depthts:
c   --------

      REAL fz,rk,fz1,rk1,rk2
      fz(rk)=fz1*(dalph_soil**rk-1.)/(dalph_soil-1.)
c
      IF (firstcall) THEN

c-----------------------------------------------------------------------
c   ground levels 
c   grnd=z/l where l is the skin depth of the diurnal cycle:
c   --------------------------------------------------------

         min_period=1800. ! en secondes
         dalph_soil=2.    ! rapport entre les epaisseurs de 2 couches succ.

         OPEN(99,file='soil.def',status='old',form='formatted',err=9999)
         READ(99,*) min_period
         READ(99,*) dalph_soil
         PRINT*,'Discretization for the soil model'
         PRINT*,'First level e-folding depth',min_period,
     s   '   dalph',dalph_soil
         CLOSE(99)
9999     CONTINUE

c   la premiere couche represente un dixieme de cycle diurne
         fz1=sqrt(min_period/3.14)

         DO jk=1,nsoilmx
            rk1=jk
            rk2=jk-1
            dz2(jk)=fz(rk1)-fz(rk2)
         ENDDO
         DO jk=1,nsoilmx-1
            rk1=jk+.5
            rk2=jk-.5
            dz1(jk)=1./(fz(rk1)-fz(rk2))
         ENDDO
         lambda=fz(.5)*dz1(1)
         PRINT*,'full layers, intermediate layers (seconds)'
         DO jk=1,nsoilmx
            rk=jk
            rk1=jk+.5
            rk2=jk-.5
            PRINT *,'fz=',
     .               fz(rk1)*fz(rk2)*3.14,fz(rk)*fz(rk)*3.14
         ENDDO
c
         firstcall =.false.
c
      ENDIF !--not firstcall
c
      IF (indice.EQ.1) THEN ! avant la couche limite
c
      nsrf = is_sic
      DO ig = 1, klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         ztherm_i(ig,nsrf)   = iice
         IF (snow(ig,nsrf).GT.0.0) ztherm_i(ig,nsrf)   = isno
      ENDIF
      ENDDO
c
      nsrf = is_lic
      DO ig = 1, klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         ztherm_i(ig,nsrf)   = iice
         IF (snow(ig,nsrf).GT.0.0) ztherm_i(ig,nsrf)   = isno
      ENDIF
      ENDDO
c
      nsrf = is_ter
      DO ig = 1, klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         ztherm_i(ig,nsrf)   = isol
         IF (snow(ig,nsrf).GT.0.0) ztherm_i(ig,nsrf)   = isno
      ENDIF
      ENDDO
c
      nsrf = is_oce
      DO ig = 1, klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         ztherm_i(ig,nsrf)   = iice
      ENDIF
      ENDDO
c-----------------------------------------------------------------------
c   Computation of the Cgrd and Dgrd coefficient for the current step:
c   ---------------------------------------------------------------

      DO jk=1,nsoilmx
         zdz2(jk)=dz2(jk)/ptimestep
      ENDDO

      DO 88888 nsrf = 1, nbsrf
c
      DO ig=1,klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         z1=zdz2(nsoilmx)+dz1(nsoilmx-1)
         zc(ig,nsoilmx-1,nsrf)=zdz2(nsoilmx)
     s                   *ptsoil(ig,nsoilmx,nsrf)/z1
         zd(ig,nsoilmx-1,nsrf)=dz1(nsoilmx-1)/z1
      ENDIF
      ENDDO

      DO jk=nsoilmx-1,2,-1
      DO ig=1,klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         z1=1./(zdz2(jk)+dz1(jk-1)+dz1(jk)*(1.-zd(ig,jk,nsrf)))
         zc(ig,jk-1,nsrf)=
     s   (ptsoil(ig,jk,nsrf)*zdz2(jk)+dz1(jk)*zc(ig,jk,nsrf))*z1
         zd(ig,jk-1,nsrf)=dz1(jk-1)*z1
      ENDIF
      ENDDO
      ENDDO

c-----------------------------------------------------------------------
c   computation of the surface diffusive flux from ground and
c   calorific capacity of the ground:
c   ---------------------------------

      DO ig=1,klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         pfluxgrd(ig,nsrf)=ztherm_i(ig,nsrf)*dz1(1)*
     s   (zc(ig,1,nsrf)+(zd(ig,1,nsrf)-1.)*ptsoil(ig,1,nsrf))
         pcapcal(ig,nsrf)=ztherm_i(ig,nsrf)*
     s   (dz2(1)+ptimestep*(1.-zd(ig,1,nsrf))*dz1(1))
         z1=lambda*(1.-zd(ig,1,nsrf))+1.
         pcapcal(ig,nsrf)=pcapcal(ig,nsrf)/z1
         pfluxgrd(ig,nsrf)=pfluxgrd(ig,nsrf)
     s   +pcapcal(ig,nsrf)*(ptsoil(ig,1,nsrf)*z1
     s   -lambda*zc(ig,1,nsrf)-ptsrf(ig,nsrf))
     s   /ptimestep
      ENDIF
      ENDDO
88888 CONTINUE
c
      ELSE IF (indice.EQ.2) THEN ! apres la couche limite
c
c-----------------------------------------------------------------------
c  Computation of the soil temperatures (for the next step)
c  using the Cgrd and Dgrd coefficient computed at the current step:
c  -----------------------------------------------
      DO 99999 nsrf = 1, nbsrf
c
c    surface temperature
      DO ig=1,klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         ptsoil(ig,1,nsrf)=(lambda*zc(ig,1,nsrf)+ptsrf(ig,nsrf))/
     s   (lambda*(1.-zd(ig,1,nsrf))+1.)
      ENDIF
      ENDDO

c   other temperatures
      DO jk=1,nsoilmx-1
      DO ig=1,klon
      IF (pctsrf(ig,nsrf).GT.epsfra) THEN
         ptsoil(ig,jk+1,nsrf)=zc(ig,jk,nsrf)
     s                       +zd(ig,jk,nsrf)*ptsoil(ig,jk,nsrf)
      ENDIF
      ENDDO
      ENDDO
c
99999 CONTINUE
c
      ELSE
c
      PRINT*, "1 (avant couche limite) ou 2 (apres) permi !", indice
      CALL abort
c
      ENDIF
c
      RETURN
      END
